@@ -934,6 +934,23 @@ static int s_generate_uuid_to_buf(struct aws_allocator *allocator, struct aws_by
     return AWS_OP_SUCCESS;
 }
 
+static struct aws_string *s_check_or_get_with_env(
+    struct aws_allocator *allocator,
+    const struct aws_string *env_key,
+    struct aws_byte_cursor option) {
+
+    AWS_ASSERT(allocator);
+    struct aws_string *out = NULL;
+
+    if (option.len) {
+        out = aws_string_new_from_cursor(allocator, &option);
+    } else {
+        aws_get_environment_value(allocator, env_key, &out);
+    }
+
+    return out;
+}
+
 static void s_check_or_get_with_profile_config(
     struct aws_allocator *allocator,
     const struct aws_profile *profile,
@@ -967,7 +984,7 @@ static void s_parameters_destroy(struct sts_web_identity_parameters *parameters)
 
 static struct sts_web_identity_parameters *s_parameters_new(
     struct aws_allocator *allocator,
-    struct aws_profile_collection *config_profile_collection_cached) {
+    const struct aws_credentials_provider_sts_web_identity_options *options) {
 
     struct sts_web_identity_parameters *parameters =
         aws_mem_calloc(allocator, 1, sizeof(struct sts_web_identity_parameters));
@@ -977,20 +994,18 @@ static struct sts_web_identity_parameters *s_parameters_new(
     parameters->allocator = allocator;
 
     bool success = false;
-    struct aws_string *region = NULL;
-    struct aws_string *role_arn = NULL;
-    struct aws_string *role_session_name = NULL;
-    struct aws_string *token_file_path = NULL;
-
-    /* check environment variables */
-    aws_get_environment_value(allocator, s_region_env, &region);
-    aws_get_environment_value(allocator, s_role_arn_env, &role_arn);
-    aws_get_environment_value(allocator, s_role_session_name_env, &role_session_name);
-    aws_get_environment_value(allocator, s_token_file_path_env, &token_file_path);
+    struct aws_string *region = s_check_or_get_with_env(allocator, s_region_env, options->region);
+    struct aws_string *role_arn = s_check_or_get_with_env(allocator, s_role_arn_env, options->role_arn);
+    struct aws_string *role_session_name =
+        s_check_or_get_with_env(allocator, s_role_session_name_env, options->role_session_name);
+    struct aws_string *token_file_path =
+        s_check_or_get_with_env(allocator, s_token_file_path_env, options->token_file_path);
+    ;
 
     /**
      * check config profile if either region, role_arn or token_file_path or role_session_name is not resolved from
-     * environment variable. Role session name can also be generated by us using uuid if not found from both sources.
+     * environment variable. Role session name can also be generated by us using uuid if not found from both
+     * sources.
      */
     struct aws_profile_collection *config_profile = NULL;
     struct aws_string *profile_name = NULL;
@@ -998,9 +1013,9 @@ static struct sts_web_identity_parameters *s_parameters_new(
     bool get_all_parameters =
         (region && region->len && role_arn && role_arn->len && token_file_path && token_file_path->len);
     if (!get_all_parameters) {
-        if (config_profile_collection_cached) {
+        if (options->config_profile_collection_cached) {
             /* Use cached profile collection */
-            config_profile = aws_profile_collection_acquire(config_profile_collection_cached);
+            config_profile = aws_profile_collection_acquire(options->config_profile_collection_cached);
         } else {
             /* Load profile collection from files */
             config_profile = s_load_profile(allocator);
@@ -1009,10 +1024,8 @@ static struct sts_web_identity_parameters *s_parameters_new(
             }
         }
 
-        profile_name = aws_get_profile_name(allocator, NULL);
-        if (profile_name) {
-            profile = aws_profile_collection_get_profile(config_profile, profile_name);
-        }
+        profile_name = aws_get_profile_name(allocator, &options->profile_name_override);
+        profile = aws_profile_collection_get_profile(config_profile, profile_name);
 
         if (!profile) {
             AWS_LOGF_ERROR(
@@ -1088,8 +1101,7 @@ struct aws_credentials_provider *aws_credentials_provider_new_sts_web_identity(
     struct aws_allocator *allocator,
     const struct aws_credentials_provider_sts_web_identity_options *options) {
 
-    struct sts_web_identity_parameters *parameters =
-        s_parameters_new(allocator, options->config_profile_collection_cached);
+    struct sts_web_identity_parameters *parameters = s_parameters_new(allocator, options);
     if (!parameters) {
         return NULL;
     }
