@@ -117,9 +117,6 @@ struct aws_s3_meta_request_vtable {
 
     /* Pause the given request */
     int (*pause)(struct aws_s3_meta_request *meta_request, struct aws_s3_meta_request_resume_token **resume_token);
-
-    /* Get the type of the aws_s3_request */
-    int (*get_request_type)(const struct aws_s3_request *request);
 };
 
 /**
@@ -178,6 +175,7 @@ struct aws_s3_meta_request {
     aws_s3_meta_request_finish_fn *finish_user_callback_after_checksum;
 
     enum aws_s3_meta_request_type type;
+    struct aws_string *s3express_session_host;
 
     struct {
         struct aws_mutex lock;
