@@ -171,6 +171,11 @@ static void s_s3_test_meta_request_finish(
             &meta_request_test_results->error_response_body, tester->allocator, result->error_response_body);
     }
 
+    if (result->error_response_operation_name != NULL) {
+        meta_request_test_results->error_response_operation_name =
+            aws_string_new_from_string(tester->allocator, result->error_response_operation_name);
+    }
+
     meta_request_test_results->finished_response_status = result->response_status;
     meta_request_test_results->finished_error_code = result->error_code;
 
@@ -390,6 +395,8 @@ int aws_s3_tester_init(struct aws_allocator *allocator, struct aws_s3_tester *te
         tester->client_bootstrap = aws_client_bootstrap_new(allocator, &bootstrap_options);
     }
 
+    tester->anonymous_creds = aws_credentials_new_anonymous(allocator);
+    tester->anonymous_signing_config.credentials = tester->anonymous_creds;
 #ifndef BYO_CRYPTO
     /* Setup the credentials provider */
     {
@@ -397,10 +404,13 @@ int aws_s3_tester_init(struct aws_allocator *allocator, struct aws_s3_tester *te
         AWS_ZERO_STRUCT(credentials_config);
         credentials_config.bootstrap = tester->client_bootstrap;
         tester->credentials_provider = aws_credentials_provider_new_chain_default(allocator, &credentials_config);
+        aws_s3_init_default_signing_config(
+            &tester->default_signing_config, g_test_s3_region, tester->credentials_provider);
     }
+#else
+    { tester->default_signing_config = tester->anonymous_signing_config; }
 #endif
 
-    aws_s3_init_default_signing_config(&tester->default_signing_config, g_test_s3_region, tester->credentials_provider);
     return AWS_OP_SUCCESS;
 
 condition_variable_failed:
@@ -420,15 +430,21 @@ int aws_s3_tester_bind_client(struct aws_s3_tester *tester, struct aws_s3_client
     ASSERT_TRUE(config->client_bootstrap == NULL);
     config->client_bootstrap = tester->client_bootstrap;
 
-    if (flags & AWS_S3_TESTER_BIND_CLIENT_REGION) {
-        ASSERT_TRUE(config->region.len == 0);
-        config->region = g_test_s3_region;
-    }
-
     if (flags & AWS_S3_TESTER_BIND_CLIENT_SIGNING) {
         ASSERT_TRUE(config->signing_config == NULL);
         config->signing_config = &tester->default_signing_config;
     }
+    if (flags & AWS_S3_TESTER_BIND_CLIENT_REGION) {
+        ASSERT_TRUE(config->region.len == 0);
+        config->region = g_test_s3_region;
+    } else {
+        if (config->signing_config) {
+            config->signing_config->region = config->region;
+        }
+    }
+    if (!config->signing_config) {
+        config->signing_config = &tester->anonymous_signing_config;
+    }
 
     ASSERT_TRUE(config->shutdown_callback == NULL);
     config->shutdown_callback = s_s3_test_client_shutdown;
@@ -496,6 +512,7 @@ void aws_s3_meta_request_test_results_clean_up(struct aws_s3_meta_request_test_r
 
     aws_http_headers_release(test_meta_request->error_response_headers);
     aws_byte_buf_clean_up(&test_meta_request->error_response_body);
+    aws_string_destroy(test_meta_request->error_response_operation_name);
     aws_http_headers_release(test_meta_request->response_headers);
     while (aws_array_list_length(&test_meta_request->synced_data.metrics) > 0) {
         struct aws_s3_request_metrics *metrics = NULL;
@@ -682,6 +699,8 @@ void aws_s3_tester_clean_up(struct aws_s3_tester *tester) {
     aws_string_destroy(tester->bucket_name);
     aws_string_destroy(tester->public_bucket_name);
 
+    aws_credentials_release(tester->anonymous_creds);
+
     aws_array_list_clean_up(&tester->client_vtable_patches);
     aws_array_list_clean_up(&tester->meta_request_vtable_patches);
 
@@ -787,6 +806,7 @@ static void s_s3_mock_client_start_destroy(void *user_data) {
     struct aws_s3_client *client = user_data;
     AWS_ASSERT(client);
 
+    aws_s3_buffer_pool_destroy(client->buffer_pool);
     aws_mem_release(client->allocator, client);
 }
 
@@ -795,6 +815,7 @@ struct aws_s3_client *aws_s3_tester_mock_client_new(struct aws_s3_tester *tester
     struct aws_s3_client *mock_client = aws_mem_calloc(allocator, 1, sizeof(struct aws_s3_client));
 
     mock_client->allocator = allocator;
+    mock_client->buffer_pool = aws_s3_buffer_pool_new(allocator, MB_TO_BYTES(8), GB_TO_BYTES(1));
     mock_client->vtable = &g_aws_s3_client_mock_vtable;
 
     aws_ref_count_init(
@@ -1394,6 +1415,7 @@ int aws_s3_tester_send_meta_request_with_options(
 
     struct aws_s3_meta_request_options meta_request_options = {
         .type = options->meta_request_type,
+        .operation_name = options->default_type_options.operation_name,
         .message = options->message,
         .checksum_config = &checksum_config,
         .resume_token = options->put_options.resume_token,
@@ -1406,6 +1428,12 @@ int aws_s3_tester_send_meta_request_with_options(
     if (options->signing_config) {
         meta_request_options.signing_config = options->signing_config;
     }
+    struct aws_signing_config_aws signing_config = {
+        .algorithm = AWS_SIGNING_ALGORITHM_V4_S3EXPRESS,
+        .service = g_s3express_service_name,
+    };
+    meta_request_options.signing_config =
+        options->use_s3express_signing ? &signing_config : meta_request_options.signing_config;
 
     struct aws_byte_buf input_stream_buffer;
     AWS_ZERO_STRUCT(input_stream_buffer);
@@ -1828,6 +1856,7 @@ int aws_s3_tester_validate_get_object_results(
 
     ASSERT_TRUE(meta_request_test_results->error_response_headers == NULL);
     ASSERT_TRUE(meta_request_test_results->error_response_body.len == 0);
+    ASSERT_NULL(meta_request_test_results->error_response_operation_name);
 
     struct aws_byte_cursor sse_byte_cursor;
 
@@ -1965,6 +1994,7 @@ int aws_s3_tester_validate_put_object_results(
 
     ASSERT_TRUE(meta_request_test_results->error_response_headers == NULL);
     ASSERT_TRUE(meta_request_test_results->error_response_body.len == 0);
+    ASSERT_NULL(meta_request_test_results->error_response_operation_name);
 
     struct aws_byte_cursor etag_byte_cursor;
     AWS_ZERO_STRUCT(etag_byte_cursor);
@@ -2021,6 +2051,35 @@ int aws_s3_tester_get_content_length(const struct aws_http_headers *headers, uin
     return AWS_OP_SUCCESS;
 }
 
+/* The default mock response is:
+<?xml version="1.0" encoding="UTF-8"?>
+<CreateSessionResult xmlns="http://s3.amazonaws.com/doc/2006-03-01/">
+  <Credentials>
+    <SessionToken>sessionToken</SessionToken>
+    <SecretAccessKey>secretKey</SecretAccessKey>
+    <AccessKeyId>accessKeyId</AccessKeyId>
+    <Expiration>2023-06-26T17:33:30Z</Expiration>
+  </Credentials>
+</CreateSessionResult>
+ */
+int aws_s3_tester_check_s3express_creds_for_default_mock_response(struct aws_credentials *credentials) {
+
+    struct aws_byte_cursor result;
+    bool match = true;
+    result = aws_credentials_get_access_key_id(credentials);
+    match &= aws_byte_cursor_eq_c_str(&result, "accessKeyId");
+    result = aws_credentials_get_secret_access_key(credentials);
+    match &= aws_byte_cursor_eq_c_str(&result, "secretKey");
+    result = aws_credentials_get_session_token(credentials);
+    match &= aws_byte_cursor_eq_c_str(&result, "sessionToken");
+
+    uint64_t expiration_secs = aws_credentials_get_expiration_timepoint_seconds(credentials);
+    ASSERT_UINT_EQUALS(1687800810, expiration_secs);
+
+    ASSERT_TRUE(match);
+    return AWS_OP_SUCCESS;
+}
+
 struct aws_string *aws_s3_tester_create_file(
     struct aws_allocator *allocator,
     struct aws_byte_cursor test_object_path,
