@@ -28,13 +28,17 @@
  * 700.  In glibc 2.11 and earlier, one obtains the definitions by
  * defining _GNU_SOURCE.
  *
- * # Relavent Links
+ * We use two feature probes to detect the need to perform this workaround.
+ * It is only applied if we can't get CLOEXEC without it and the build doesn't
+ * fail with _XOPEN_SOURCE being defined.
+ *
+ * # Relevent Links
  *
  * - POSIX.1-2017: https://pubs.opengroup.org/onlinepubs/9699919799
  * - https://stackoverflow.com/a/5724485
  * - https://stackoverflow.com/a/5583764
  */
-#ifndef _XOPEN_SOURCE
+#if !defined(S2N_CLOEXEC_SUPPORTED) && defined(S2N_CLOEXEC_XOPEN_SUPPORTED) && !defined(_XOPEN_SOURCE)
     #define _XOPEN_SOURCE 700
     #include <fcntl.h>
     #undef _XOPEN_SOURCE
@@ -65,6 +69,7 @@
 #include "error/s2n_errno.h"
 #include "stuffer/s2n_stuffer.h"
 #include "utils/s2n_fork_detection.h"
+#include "utils/s2n_init.h"
 #include "utils/s2n_mem.h"
 #include "utils/s2n_random.h"
 #include "utils/s2n_result.h"
@@ -174,7 +179,9 @@ S2N_RESULT s2n_get_mix_entropy(struct s2n_blob *blob)
 /* Deletes pthread key at process-exit */
 static void __attribute__((destructor)) s2n_drbg_rand_state_key_cleanup(void)
 {
-    (void) pthread_key_delete(s2n_per_thread_rand_state_key);
+    if (s2n_is_initialized()) {
+        pthread_key_delete(s2n_per_thread_rand_state_key);
+    }
 }
 
 static void s2n_drbg_destructor(void *_unused_argument)
@@ -542,7 +549,9 @@ S2N_RESULT s2n_rand_cleanup_thread(void)
     s2n_per_thread_rand_state.drbgs_initialized = false;
 
     /* Unset the thread-local destructor */
-    pthread_setspecific(s2n_per_thread_rand_state_key, NULL);
+    if (s2n_is_initialized()) {
+        pthread_setspecific(s2n_per_thread_rand_state_key, NULL);
+    }
 
     return S2N_RESULT_OK;
 }
