@@ -363,7 +363,7 @@ static void s_lifecycle_event_connection_success(
     result = PyObject_CallMethod(
         client->client_core,
         "_on_lifecycle_connection_success",
-        "(OiOIOHOiOOOIs#s#OOOOOOOOHs#s#iIHIHHHOOOOO)",
+        "(OiOIOHOiOOOIs#OIs#OOOOOOOOHs#s#iIHIHHHOOOOO)",
         /* connack packet  */
         /* O */ connack->session_present ? Py_True : Py_False,
         /* i */ (int)connack->reason_code,
@@ -379,6 +379,8 @@ static void s_lifecycle_event_connection_success(
         /* I */ (unsigned int)(connack->maximum_packet_size ? *connack->maximum_packet_size : 0),
         /* s */ connack->assigned_client_identifier ? connack->assigned_client_identifier->ptr : NULL,
         /* # */ connack->assigned_client_identifier ? connack->assigned_client_identifier->len : 0,
+        /* O */ connack->topic_alias_maximum ? Py_True : Py_False,
+        /* I */ (unsigned int)(connack->topic_alias_maximum ? *connack->topic_alias_maximum : 0),
         /* s */ connack->reason_string ? connack->reason_string->ptr : NULL,
         /* # */ connack->reason_string ? connack->reason_string->len : 0,
         /* O */ user_property_count > 0 ? user_properties_list : Py_None,
@@ -729,6 +731,69 @@ PyObject *aws_py_mqtt5_ws_handshake_transform_complete(PyObject *self, PyObject
     Py_RETURN_NONE;
 }
 
+static bool s_py_topic_aliasing_options_init(
+    struct aws_mqtt5_client_topic_alias_options *topic_aliasing_options,
+    PyObject *py_topic_aliasing_options) {
+    AWS_ZERO_STRUCT(*topic_aliasing_options);
+
+    bool success = false;
+    PyObject *py_outbound_behavior = PyObject_GetAttrString(py_topic_aliasing_options, "outbound_behavior");
+    PyObject *py_outbound_cache_max_size = PyObject_GetAttrString(py_topic_aliasing_options, "outbound_cache_max_size");
+    PyObject *py_inbound_behavior = PyObject_GetAttrString(py_topic_aliasing_options, "inbound_behavior");
+    PyObject *py_inbound_cache_max_size = PyObject_GetAttrString(py_topic_aliasing_options, "inbound_cache_max_size");
+
+    if (py_outbound_behavior != NULL && !PyObject_GetAsOptionalIntEnum(
+                                            py_outbound_behavior,
+                                            "TopicAliasingOptions",
+                                            "outbound_behavior",
+                                            (int *)&topic_aliasing_options->outbound_topic_alias_behavior)) {
+        if (PyErr_Occurred()) {
+            goto done;
+        }
+    }
+
+    if (py_outbound_cache_max_size != NULL && !PyObject_GetAsOptionalUint16(
+                                                  py_outbound_cache_max_size,
+                                                  "TopicAliasingOptions",
+                                                  "outbound_cache_max_size",
+                                                  &topic_aliasing_options->outbound_alias_cache_max_size)) {
+        if (PyErr_Occurred()) {
+            goto done;
+        }
+    }
+
+    if (py_inbound_behavior != NULL && !PyObject_GetAsOptionalIntEnum(
+                                           py_inbound_behavior,
+                                           "TopicAliasingOptions",
+                                           "inbound_behavior",
+                                           (int *)&topic_aliasing_options->inbound_topic_alias_behavior)) {
+        if (PyErr_Occurred()) {
+            goto done;
+        }
+    }
+
+    if (py_inbound_cache_max_size != NULL && !PyObject_GetAsOptionalUint16(
+                                                 py_inbound_cache_max_size,
+                                                 "TopicAliasingOptions",
+                                                 "inbound_cache_max_size",
+                                                 &topic_aliasing_options->inbound_alias_cache_size)) {
+        if (PyErr_Occurred()) {
+            goto done;
+        }
+    }
+
+    success = true;
+
+done:
+
+    Py_XDECREF(py_outbound_behavior);
+    Py_XDECREF(py_outbound_cache_max_size);
+    Py_XDECREF(py_inbound_behavior);
+    Py_XDECREF(py_inbound_cache_max_size);
+
+    return success;
+}
+
 /*******************************************************************************
  * Client Init
  ******************************************************************************/
@@ -782,13 +847,14 @@ PyObject *aws_py_mqtt5_client_new(PyObject *self, PyObject *args) {
     PyObject *min_connected_time_to_reset_reconnect_delay_ms_py; /* optional uint64_t */
     PyObject *ping_timeout_ms_py;                                /* optional uint32_t */
     PyObject *ack_timeout_seconds_py;                            /* optional uint32_t */
+    PyObject *topic_aliasing_options_py;                         /* optional TopicAliasingOptions */
     /* Callbacks */
     PyObject *is_websocket_none_py;
     PyObject *client_core_py;
 
     if (!PyArg_ParseTuple(
             args,
-            "Os#HOOOOz#Oz#z#OOOOOOOOOz*Oz#OOOz#z*z#OOOOOOOOOOOO",
+            "Os#HOOOOz#Oz#z#OOOOOOOOOz*Oz#OOOz#z*z#OOOOOOOOOOOOO",
             /* O */ &self_py,
             /* s */ &host_name.ptr,
             /* # */ &host_name.len,
@@ -839,6 +905,7 @@ PyObject *aws_py_mqtt5_client_new(PyObject *self, PyObject *args) {
             /* O */ &min_connected_time_to_reset_reconnect_delay_ms_py,
             /* O */ &ping_timeout_ms_py,
             /* O */ &ack_timeout_seconds_py,
+            /* O */ &topic_aliasing_options_py,
 
             /* O */ &is_websocket_none_py,
             /* O */ &client_core_py)) {
@@ -871,6 +938,7 @@ PyObject *aws_py_mqtt5_client_new(PyObject *self, PyObject *args) {
     AWS_ZERO_STRUCT(will_options);
 
     struct aws_http_proxy_options proxy_options;
+    struct aws_mqtt5_client_topic_alias_options topic_aliasing_options;
 
     struct aws_tls_ctx *tls_ctx = NULL;
 
@@ -1021,6 +1089,13 @@ PyObject *aws_py_mqtt5_client_new(PyObject *self, PyObject *args) {
         goto done;
     }
 
+    if (topic_aliasing_options_py != Py_None) {
+        if (!s_py_topic_aliasing_options_init(&topic_aliasing_options, topic_aliasing_options_py)) {
+            goto done;
+        }
+        client_options.topic_aliasing_options = &topic_aliasing_options;
+    }
+
     /* CONNECT OPTIONS */
 
     connect_options.client_id = client_id;
@@ -1129,7 +1204,6 @@ PyObject *aws_py_mqtt5_client_new(PyObject *self, PyObject *args) {
     int will_payload_format_tmp = 0;
     enum aws_mqtt5_payload_format_indicator will_payload_format_enum_tmp = 0;
     uint32_t will_message_expiry_interval_seconds_tmp = 0;
-    uint16_t will_topic_alias_tmp = 0;
     struct aws_byte_cursor will_correlation_data_tmp;
     if (!PyObject_IsTrue(is_will_none_py)) {
         will.qos = PyObject_GetIntEnum(will_qos_val_py, AWS_PYOBJECT_KEY_QOS);
@@ -1160,12 +1234,6 @@ PyObject *aws_py_mqtt5_client_new(PyObject *self, PyObject *args) {
             goto done;
         }
 
-        will.topic_alias = PyObject_GetAsOptionalUint16(
-            will_topic_alias_py, AWS_PYOBJECT_KEY_WILL_PACKET, AWS_PYOBJECT_KEY_TOPIC_ALIAS, &will_topic_alias_tmp);
-        if (PyErr_Occurred()) {
-            goto done;
-        }
-
         if (will_response_topic.ptr) {
             will.response_topic = &will_response_topic;
         }
